/* ========================================================= */
/*  Loading & Saving Data version xavier                     */
/*                                                           */
/* ========================================================= */

/* == Include Standard ===================================== */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/* == Include Fusion  ====================================== */

#include "fusion-c/header/msx_fusion.h"
#include "fusion-c/header/vars_msxSystem.h"
#include "fusion-c/header/vdp_sprites.h"

/* ==  Include Perso  ====================================== */

/* == DEFINE =============================================== */

#define  HALT __asm halt __endasm     // wait for the next interrupt

#define  AFB_SIZE 3991                // taille du fichier de la banque des sons 
#define  NBR_PISTE 15                 // nombre de sons dans la banque des sons

/* == Constantes globales ================================== */



/* == Variables globales =================================== */

static FCB file;    			                       // Initialisation of the file System Structure variable (Mandatory)

/* ========================================================= */
/* ==  Fonctions Diverses ================================== */
/* ========================================================= */

/* --------------------------------------------------------- */

void wait(int cicles)

/* --------------------------------------------------------- */
/* --  Fait une pausse de quelques milli-secondes  --------- */ 
/* --------------------------------------------------------- */
/* --Generates a pause in the execution of n interruptions-- */
/* --PAL: 50=1second. ; NTSC: 60=1second-------------------- */
/* --------------------------------------------------------- */
{
  int i;
  for(i=0;i<cicles;i++) HALT;
  return;
}

/* --------------------------------------------------------- */

void waitspace( void)

/* --------------------------------------------------------- */
/* --  Fait une pausse de quelques milli-secondes  --------- */ 
/* --------------------------------------------------------- */
/* --Generates a pause in the execution of n interruptions-- */
/* --PAL: 50=1second. ; NTSC: 60=1second-------------------- */
/* --------------------------------------------------------- */
{
 while (TriggerRead (0)==0) wait (1);
 while (TriggerRead (0)!=0) wait (1);
}

/* --------------------------------------------------------- */

void FT_SetName( FCB *p_fcb, const char *p_name )

/* --------------------------------------------------------- */
/* -- Set the name of a file to load (MSX DOS) ------------- */
/* --------------------------------------------------------- */
{
  char i, j;
  memset( p_fcb, 0, sizeof(FCB) );

  for( i = 0; i < 11; i++ )
  {
    p_fcb->name[i] = ' ';
  }

  for( i = 0; (i < 8) && (p_name[i] != 0) && (p_name[i] != '.'); i++ )
  {
    p_fcb->name[i] =  p_name[i];
  }

  if( p_name[i] == '.' )
  {
    i++;
    for( j = 0; (j < 3) && (p_name[i + j] != 0) && (p_name[i + j] != '.'); j++ )
    {
      p_fcb->ext[j] =  p_name[i + j] ;
    }
  }
}

/* --------------------------------------------------------- */

void FT_LoadData(char *file_name, char *buffer, int size, int skip) 

/* --------------------------------------------------------- */
/* -- Load Data to a specific pointer ---------------------- */
/* -- size is the size of data to read --------------------- */
/* -- skip represent the number of Bytes ------------------- */
/* -- you want to skip from the begining of the file ------- */
/* -- Example: skip=7 to skip 7 bytes of a MSX bin --------- */
/* --------------------------------------------------------- */
{
  FT_SetName( &file, file_name );

  fcb_open( &file );

  if (skip>0) fcb_read( &file, buffer, skip );
    
  fcb_read( &file, buffer, size );
  
  fcb_close( &file );  
}


/* ========================================================= */
/* ==  Fonctions Gestion des Fonts ========================= */
/* ========================================================= */

/* --------------------------------------------------------- */

void FT_LoadFonts(char *file_name)

/* --------------------------------------------------------- */
/* -- This function is loading Fonts ----------------------- */
/* --------------------------------------------------------- */
{
  unsigned int  start_adr_fonts;                          // adresse de la table des charactere (TGP) dans la VRAM
  unsigned int  taille_buff_fonts=2048;                   // taille de la table des characteres (TNP) 16*16 (16 lignes de 16 caractères)
  unsigned char buff_fonts[2048];                         // Buffer Array of 2048 bytes. The size of the whole Fonts data
  
  start_adr_fonts=Peekw(0xF3C1);                          // lit adresse de la table des charactere (TGP) dans la VRAM
 
  FT_LoadData(file_name, buff_fonts, taille_buff_fonts, 0);

  CopyRamToVram(&buff_fonts, start_adr_fonts, taille_buff_fonts);
}


/* -------------------------------------------------------------- */

void FT_ChgColorFonts ( void)

/* -------------------------------------------------------------- */
/* -- Change toutes les couleurs des caractères (Block de 8)   -- */ 
/* -- code couleur : 4bits fort la couleur du caractere        -- */ 
/* --                4bits faible la couleur du fond           -- */
/* -------------------------------------------------------------- */
{
  unsigned int start_adr_ColorFonts=0x2000;   // adresse de la table des couleurs des charateres (TC) dans la VRAM
  unsigned int taille_ColorFonts=32;          // taille de la table des couleurs des charateres (TC) dans la VRAM
 
  // FONTS COLORS DATA
  unsigned char COLORS_DATA[32]=
    {
    0x1F,0x1F,0x1F,0x1F,0x1F,0x1F,0x1F,0x1F,
    0x1F,0x1F,0x1F,0x1F,0x1F,0x1F,0x1F,0x1F,
    0x1F,0x1F,0x1F,0x1F,0x1F,0x1F,0x1F,0x1F,
    0x1F,0x1F,0x1F,0x1F,0x1F,0x1F,0x1F,0x1F,
    };

  CopyRamToVram( &COLORS_DATA, start_adr_ColorFonts, taille_ColorFonts );
}


/* --------------------------------------------------------- */

void FT_AffFonts( void)

/* --------------------------------------------------------- */
/* -- This function is loading Fonts ----------------------- */
/* --------------------------------------------------------- */
{
  int i;  
  int j;
  Locate (0,0);
  for (i=0;i<16;i++)
  {
    for (j=0;j<16;j++)  
      printf("%c",i*16+j);
    printf("\n\r");
  }
}

/* --------------------------------------------------------- */

/* ========================================================= */
/* ==  Fonctions Gestion des Tiles ========================= */
/* ========================================================= */

/* --------------------------------------------------------- */

void FT_LoadTiles(char *file_name)

/* --------------------------------------------------------- */
/* -- This function is loading Tiles ----------------------- */
/* --------------------------------------------------------- */
{
  unsigned int  start_adr_tiles=0x1800;                  // adresse de la table des positions des characteres (TNP) dans la VRAM 
  unsigned int  taille_buff_tiles=768;                   // taille de la table des positions des characteres (TNP) 24 lignes de 32 colonnes (24*32)
  unsigned char buff_tiles[768];                         // Buffer Array of 768 bytes. The size of the whole Tiles data
 
  FT_LoadData(file_name, buff_tiles, taille_buff_tiles, 0);

  CopyRamToVram(&buff_tiles, start_adr_tiles, taille_buff_tiles);
}


/* --------------------------------------------------------- */

/* ========================================================= */
/* ==  Programme Principal ================================= */
/* ========================================================= */

void main(void )
{

/* ==  Variables Diverses  ================================= */

/* ==  Variables   ========================================= */

/* ==  Initialisation MSX1  ================================ */

	if(ReadMSXtype()==3)  // IF MSX is Turbo-R Switch CPU to Z80 Mode
  	{
  		ChangeCPU(0);
	}
  
/* == Initialisation Ecran  ================================ */

 	Screen(1);
 	SetColors (15,1,14);

/* == Corps Principal  ===================================== */


/* -- Initialisation Ecran --------------------------------- */  

  SpriteOn ();
  FT_AffFonts()  ;
  FT_ChgColorFonts ();
  FT_LoadFonts ("fbg.xav");


/* --------------------------------------------------------- */ 

  SpriteOn ();
  waitspace ();
  SpriteOff ();
  waitspace ();
    
/* --------------------------------------------------------- */ 

  FT_LoadTiles ("TM.xav");

/* --------------------------------------------------------- */ 

  SpriteOn ();
  waitspace ();
  SpriteOff ();
  waitspace ();
    
/* --------------------------------------------------------- */ 

  FT_LoadTiles ("TI1.xav");

/* --------------------------------------------------------- */ 

  SpriteOn ();
  waitspace ();
  SpriteOff ();
  waitspace ();
    
/* --------------------------------------------------------- */ 

  FT_LoadTiles ("TI2.xav");

/* --------------------------------------------------------- */ 

  SpriteOn ();
  waitspace ();
  SpriteOff ();
  waitspace ();
    
/* --------------------------------------------------------- */ 

 
/* == Fin Principal  ======================================= */

	Screen(0);
  SetColors ( 15, 4, 4);

/* ========================================================= */  

}

/* ========================================================= */  